# Class: ovn
# ===========================
#
# Install and configure the OVN services
#
# Authors
# -------
#
# OVN and OpenStack contributer
#
# Copyright
# ---------
#
# Apache License 2.0 (see LICENSE file)
#
class ovn {}
